@@ -13,15 +13,29 @@ from zmq.eventloop import ioloop
 from tsuru import common
 
 
+class FileWatcher(object):
+    def __init__(self, filename, callback):
+        self._filename = filename
+        self._callback = callback
+        self._mtime = 0
+
+    def __call__(self, *args, **kwargs):
+        if os.path.exists(self._filename):
+            mtime = os.path.getmtime(self._filename)
+            if mtime > self._mtime:
+                self._mtime = mtime
+                self._callback()
+
+
 class ApprcWatcher(CircusPlugin):
     name = "apprc_watcher"
 
     def __init__(self, *args, **config):
         super(ApprcWatcher, self).__init__(*args, **config)
-        self.loop_rate = config.get("loop_rate", 10)  # in seconds
+        self.loop_rate = config.get("loop_rate", 3)  # in seconds
         self.apprc = config.get("apprc", "/home/application/apprc")
         self.port = config.get("port", "8888")
-        self.period = ioloop.PeriodicCallback(self.look_after,
+        self.period = ioloop.PeriodicCallback(FileWatcher(self.apprc, self.reload_env),
                                               self.loop_rate * 1000,
                                               self.loop)
 
@@ -34,13 +48,12 @@ class ApprcWatcher(CircusPlugin):
     def handle_recv(self, data):
         pass
 
-    def look_after(self):
-        if os.path.exists(self.apprc):
-            envs = {"port": self.port}
-            envs.update(common.load_envs(self.apprc))
-            for name in self.cmds():
-                if not name.startswith("plugin:"):
-                    self.add_envs(name, envs)
+    def reload_env(self):
+        envs = {"port": self.port}
+        envs.update(common.load_envs(self.apprc))
+        for name in self.cmds():
+            if not name.startswith("plugin:"):
+                self.add_envs(name, envs)
 
     def add_envs(self, name, envs):
         current = self.call("options", name=name)["options"]["env"]
@@ -60,7 +73,7 @@ class ProcfileWatcher(CircusPlugin):
 
     def __init__(self, *args, **config):
         super(ProcfileWatcher, self).__init__(*args, **config)
-        self.loop_rate = config.get("loop_rate", 10)  # in seconds
+        self.loop_rate = config.get("loop_rate", 3)  # in seconds
         self.procfile_path = config.get("app_path",
                                         "/home/application/current/Procfile")
         self.working_dir = config.get("working_dir",
@@ -72,7 +85,8 @@ class ProcfileWatcher(CircusPlugin):
                                                   "tsuru.stream.Stream")}
         self.stdout_stream = {"class": config.get("stdout_stream",
                                                   "tsuru.stream.Stream")}
-        self.period = ioloop.PeriodicCallback(self.look_after,
+        file_watcher = FileWatcher(self.procfile_path, self.reload_procfile)
+        self.period = ioloop.PeriodicCallback(file_watcher,
                                               self.loop_rate * 1000,
                                               self.loop)
         self.circus_client = CircusClient()
@@ -130,17 +144,16 @@ class ProcfileWatcher(CircusPlugin):
                 to_change[name] = procfile.commands[name]
         return to_add, to_remove, to_change
 
-    def look_after(self):
-        if os.path.exists(self.procfile_path):
-            with open(self.procfile_path) as file:
-                procfile = Procfile(file.read())
-                to_add, to_remove, to_change = self.commands(procfile)
+    def reload_procfile(self):
+        with open(self.procfile_path) as file:
+            procfile = Procfile(file.read())
+            to_add, to_remove, to_change = self.commands(procfile)
 
-                for name in to_remove:
-                    self.remove_watcher(name)
+            for name in to_remove:
+                self.remove_watcher(name)
 
-                for name in to_add:
-                    self.add_watcher(name=name, cmd=procfile.commands[name])
+            for name in to_add:
+                self.add_watcher(name=name, cmd=procfile.commands[name])
 
-                for name, cmd in to_change.items():
-                    self.change_cmd(name=name, cmd=procfile.commands[name])
+            for name, cmd in to_change.items():
+                self.change_cmd(name=name, cmd=procfile.commands[name])
